package edu.ycp.cs340.jsonparser;

public class JSONParser {
	private Lexer lexer;
	
	public JSONParser(Lexer lexer) {
		this.lexer = lexer;
	}
	
	public Node parseValue() {
		Node value = new Node(Symbol.VALUE);
		
		Token tok = lexer.peek();
		
		if (tok == null) {
			throw new ParserException("Unexpected end of input reading value");
		}
		
		if (tok.getSymbol() == Symbol.STRING_LITERAL) {
			// Value → StringLiteral
			value.getChildren().add(expect(Symbol.STRING_LITERAL));
		} else if (tok.getSymbol() == Symbol.INT_LITERAL) {
			// Value → IntLiteral
			value.getChildren().add(expect(Symbol.INT_LITERAL));
		} else if (tok.getSymbol() == Symbol.LBRACE) {
			// Value → Object
			value.getChildren().add(parseObject());
		} else if (tok.getSymbol() == Symbol.LBRACKET) {
			// Value → Array
			value.getChildren().add(parseArray());
		} else {
			throw new ParserException("Unexpected token looking for value: " + tok);
		}
		
		return value;
	}

	private Node parseObject() {
		Node object = new Node(Symbol.OBJECT);
		
		// Object → "{" OptFieldList "}"
		object.getChildren().add(expect(Symbol.LBRACE));
		object.getChildren().add(parseOptFieldList());
		object.getChildren().add(expect(Symbol.RBRACE));
		
		return object;
	}

	private Node parseOptFieldList() {
		throw new UnsupportedOperationException("TODO -implement");
	}

	private Node parseArray() {
		Node array = new Node(Symbol.ARRAY);
		
		// TODO: implement
		
		return array;
	}

	private Node expect(Symbol symbol) {
		Token tok = lexer.next();
		if (tok.getSymbol() != symbol) {
			throw new LexerException("Unexpected token " + tok + " (was expecting " + symbol + ")");
		}
		return new Node(tok);
	}
}
